@@ -49,11 +49,13 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
 #include <afs/stds.h>
+
+#include <roken.h>
+#include <afs/opr.h>
+
 #include <ctype.h>
-#include <stdio.h>
+
 #include <lock.h>
 #include <afs/afsutil.h>
 #include <ubik.h>
@@ -61,13 +63,7 @@
 #include <rx/rx.h>
 #include <rx/rxkad.h>
 #include <afs/auth.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#endif
-#include <string.h>
+
 #include "ptserver.h"
 #include "pterror.h"
 #include "ptprototypes.h"
@@ -176,61 +172,10 @@ CreateOK(struct ubik_trans *ut, afs_int32 cid, afs_int32 oid, afs_int32 flag,
 afs_int32
 WhoIsThis(struct rx_call *acall, struct ubik_trans *at, afs_int32 *aid)
 {
-    int foreign = 0;
-    /* aid is set to the identity of the caller, if known, else ANONYMOUSID */
-    /* returns -1 and sets aid to ANONYMOUSID on any failure */
-    struct rx_connection *tconn;
-    afs_int32 code;
-    char tcell[MAXKTCREALMLEN];
-    char name[MAXKTCNAMELEN];
-    char inst[MAXKTCNAMELEN];
-    int ilen;
-    char vname[256];
-
-    *aid = ANONYMOUSID;
-    tconn = rx_ConnectionOf(acall);
-    code = rx_SecurityClassOf(tconn);
-    if (code == 0)
-	return 0;
-    else if (code == 1) {	/* vab class */
-	goto done;		/* no longer supported */
-    } else if (code == 2) {	/* kad class */
-	if ((code = rxkad_GetServerInfo(acall->conn, NULL, 0 /*was &exp */ ,
-					name, inst, tcell, NULL)))
-	    goto done;
-#if 0
-	/* This test is unnecessary, since rxkad_GetServerInfo already check.
-	 * In addition, this is wrong since exp must be unsigned. */
-	if (exp < FT_ApproxTime())
-	    goto done;
-#endif
-	if (tcell[0])
-	    foreign = afs_is_foreign_ticket_name(name,inst,tcell,pr_realmName);
-
-	strncpy(vname, name, sizeof(vname));
-	if ((ilen = strlen(inst))) {
-	    if (strlen(vname) + 1 + ilen >= sizeof(vname))
-		goto done;
-	    strcat(vname, ".");
-	    strcat(vname, inst);
-	}
-	if (foreign) {
-	    if (strlen(vname) + strlen(tcell) + 1 >= sizeof(vname))
-		goto done;
-	    strcat(vname, "@");
-	    strcat(vname, tcell);
-	}
-	if (strcmp(AUTH_SUPERUSER, vname) == 0)
-	    *aid = SYSADMINID;	/* special case for the fileserver */
-	else {
-	    lcstring(vname, vname, sizeof(vname));
-	    code = NameToID(at, vname, aid);
-	}
-    }
-  done:
-    if (code && !pr_noAuth)
-	return -1;
-    return 0;
+    int code = WhoIsThisWithName(acall, at, aid, NULL);
+    if (code == 2 && *aid == ANONYMOUSID)
+	return PRNOENT;
+    return code;
 }
 
 afs_int32
@@ -345,19 +290,13 @@ newEntry(struct rx_call *call, char aname[], afs_int32 flag, afs_int32 oid,
      * automatic id assignment.
      */
     code = WhoIsThisWithName(call, tt, cid, cname);
-    if (code && code != 2)
-	ABORT_WITH(tt, PRPERM);
-    admin = IsAMemberOf(tt, *cid, SYSADMINID);
-    if (code == 2 /* foreign cell request */) {
-	if (!restricted && (strcmp(aname, cname) == 0)) {
-	    /* can't autoregister while providing an owner id */
-	    if (oid != 0)
-		ABORT_WITH(tt, PRPERM);
-
-	    admin = 1;
-	    oid = SYSADMINID;
-	    *cid = SYSADMINID;
-	}
+    if (code != 2) {		/* 2 specifies that this is a foreign cell request */
+	if (code)
+	    ABORT_WITH(tt, PRPERM);
+	admin = IsAMemberOf(tt, *cid, SYSADMINID);
+    } else {
+	admin = ((!restricted && !strcmp(aname, cname))) || IsAMemberOf(tt, *cid, SYSADMINID);
+	oid = *cid = SYSADMINID;
     }
     if (!CreateOK(tt, *cid, oid, flag, admin))
 	ABORT_WITH(tt, PRPERM);
@@ -685,9 +624,9 @@ idToName(struct rx_call *call, idlist *aid, namelist *aname)
     size = aid->idlist_len;
     if (size == 0)
 	return 0;
-    if (size < 0 || size > INT_MAX / PR_MAXNAMELEN)
+    if (size < 0)
 	return PRTOOMANY;
-    aname->namelist_val = (prname *) calloc(size, PR_MAXNAMELEN);
+    aname->namelist_val = (prname *) malloc(size * PR_MAXNAMELEN);
     aname->namelist_len = 0;
     if (aname->namelist_val == 0)
 	return PRNOMEM;
@@ -1250,7 +1189,6 @@ getCPS2(struct rx_call *call, afs_int32 aid, afs_uint32 ahost, prlist *alist,
     afs_int32 hostid;
     int host_list = 0;
     struct in_addr iaddr;
-    char hoststr[16];
 
     *over = 0;
     iaddr.s_addr = ntohl(ahost);
@@ -1283,7 +1221,7 @@ getCPS2(struct rx_call *call, afs_int32 aid, afs_uint32 ahost, prlist *alist,
 	    || !AccessOK(tt, *cid, &tentry, PRP_MEMBER_MEM, PRP_MEMBER_ANY))
 	    ABORT_WITH(tt, PRPERM);
     }
-    code = NameToID(tt, afs_inet_ntoa_r(iaddr.s_addr, hoststr), &hostid);
+    code = NameToID(tt, inet_ntoa(iaddr), &hostid);
     if (code == PRSUCCESS && hostid != 0) {
 	temp = FindByID(tt, hostid);
 	if (temp) {
@@ -1330,7 +1268,6 @@ getHostCPS(struct rx_call *call, afs_uint32 ahost, prlist *alist,
     struct prentry host_tentry;
     afs_int32 hostid;
     struct in_addr iaddr;
-    char hoststr[16];
 
     *over = 0;
     iaddr.s_addr = ntohl(ahost);
@@ -1349,7 +1286,7 @@ getHostCPS(struct rx_call *call, afs_uint32 ahost, prlist *alist,
     if (code)
 	ABORT_WITH(tt, code);
 
-    code = NameToID(tt, afs_inet_ntoa_r(iaddr.s_addr, hoststr), &hostid);
+    code = NameToID(tt, inet_ntoa(iaddr), &hostid);
     if (code == PRSUCCESS && hostid != 0) {
 	temp = FindByID(tt, hostid);
 	if (temp) {
@@ -1647,7 +1584,6 @@ put_prentries(struct prentry *tentry, prentries *bulkentries)
     entry = (struct prlistentries *)bulkentries->prentries_val;
     entry += bulkentries->prentries_len;
 
-    memset(entry, 0, sizeof(*entry));
     entry->flags = tentry->flags >> PRIVATE_SHIFT;
     if (entry->flags == 0) {
 	entry->flags =
@@ -1662,6 +1598,7 @@ put_prentries(struct prentry *tentry, prentries *bulkentries)
     entry->nusers = tentry->nusers;
     entry->count = tentry->count;
     strncpy(entry->name, tentry->name, PR_MAXNAMELEN);
+    memset(entry->reserved, 0, sizeof(entry->reserved));
     bulkentries->prentries_len++;
     return 0;
 }
@@ -2125,12 +2062,11 @@ addWildCards(struct ubik_trans *tt, prlist *alist, afs_uint32 host)
     afs_int32 hostid;
     int size = 0, i, code;
     int added = 0;
-    char hoststr[16];
 
     while ((host = (host & wild))) {
 	wild = htonl(ntohl(wild) << 8);
 	iaddr.s_addr = host;
-	code = NameToID(tt, afs_inet_ntoa_r(iaddr.s_addr, hoststr), &hostid);
+	code = NameToID(tt, inet_ntoa(iaddr), &hostid);
 	if (code == PRSUCCESS && hostid != 0) {
 	    temp = FindByID(tt, hostid);
 	    if (temp) {
@@ -2167,6 +2103,7 @@ static afs_int32
 WhoIsThisWithName(struct rx_call *acall, struct ubik_trans *at, afs_int32 *aid,
 		  char *aname)
 {
+    int foreign = 0;
     /* aid is set to the identity of the caller, if known, else ANONYMOUSID */
     /* returns -1 and sets aid to ANONYMOUSID on any failure */
     struct rx_connection *tconn;
@@ -2185,13 +2122,13 @@ WhoIsThisWithName(struct rx_call *acall, struct ubik_trans *at, afs_int32 *aid,
     else if (code == 1) {	/* vab class */
 	goto done;		/* no longer supported */
     } else if (code == 2) {	/* kad class */
-
-	int clen;
-
-	if ((code = rxkad_GetServerInfo(acall->conn, NULL, 0 /*was &exp */ ,
+	if ((code = rxkad_GetServerInfo(acall->conn, NULL, NULL,
 					name, inst, tcell, NULL)))
 	    goto done;
 
+	if (tcell[0])
+	    foreign = afs_is_foreign_ticket_name(name, inst, tcell,
+						 pr_realmName);
 
 	strncpy(vname, name, sizeof(vname));
 	if ((ilen = strlen(inst))) {
@@ -2200,19 +2137,16 @@ WhoIsThisWithName(struct rx_call *acall, struct ubik_trans *at, afs_int32 *aid,
 	    strcat(vname, ".");
 	    strcat(vname, inst);
 	}
-	if ((clen = strlen(tcell))) {
-	    int foreign = afs_is_foreign_ticket_name(name,inst,tcell,pr_realmName);
-
-	    if (foreign) {
-		if (strlen(vname) + 1 + clen >= sizeof(vname))
-		    goto done;
-		strcat(vname, "@");
-		strcat(vname, tcell);
-		lcstring(vname, vname, sizeof(vname));
-		code = NameToID(at, vname, aid);
+	if (foreign) {
+	     if (strlen(vname) + strlen(tcell) + 1  >= sizeof(vname))
+		goto done;
+	     strcat(vname, "@");
+	     strcat(vname, tcell);
+	     lcstring(vname, vname, sizeof(vname));
+	     code = NameToID(at, vname, aid);
+	     if (aname)
 		strcpy(aname, vname);
-		return 2;
-	    }
+	     return 2;
 	}
 
 	if (strcmp(AUTH_SUPERUSER, vname) == 0)
