@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -14,26 +14,16 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
+
+#include <afs/opr.h>
 
-# include <sys/types.h>
-# include <errno.h>
-# include <signal.h>
-# include <string.h>
 #ifdef AFS_NT40_ENV
 # include <WINNT/syscfg.h>
 #else
-# include <sys/socket.h>
-# include <sys/file.h>
-# include <netdb.h>
-# include <sys/stat.h>
-# include <netinet/in.h>
-# include <sys/time.h>
 # include <net/if.h>
-# include <sys/ioctl.h>
-# include <unistd.h>
 #endif
-# include <fcntl.h>
-#if !defined(AFS_AIX_ENV) && !defined(AFS_NT40_ENV) 
+#if !defined(AFS_AIX_ENV) && !defined(AFS_NT40_ENV)
 # include <sys/syscall.h>
 #endif
 #include <afs/afs_args.h>
@@ -54,14 +44,11 @@
 #endif
 #endif
 
-#ifndef AFS_NT40_ENV
-# include <sys/time.h>
-#endif
-# include "rx.h"
-# include "rx_globals.h"
-
+#include "rx.h"
+#include "rx_atomic.h"
+#include "rx_globals.h"
+#include "rx_stats.h"
 #ifdef AFS_PTHREAD_ENV
-#include <assert.h>
 
 /*
  * The rx_if_init_mutex mutex protects the following global variables:
@@ -94,7 +81,7 @@ afs_kmutex_t rx_if_mutex;
  * Make a socket for receiving/sending IP packets.  Set it into non-blocking
  * and large buffering modes.  If port isn't specified, the kernel will pick
  * one.  Returns the socket (>= 0) on success.  Returns OSI_NULLSOCKET on
- * failure. Port must be in network byte order.	
+ * failure. Port must be in network byte order.
  */
 osi_socket
 rxi_GetHostUDPSocket(u_int ahost, u_short port)
@@ -112,7 +99,7 @@ rxi_GetHostUDPSocket(u_int ahost, u_short port)
 #endif
 #endif
 
-#if !defined(AFS_NT40_ENV) 
+#if !defined(AFS_NT40_ENV)
     if (ntohs(port) >= IPPORT_RESERVED && ntohs(port) < IPPORT_USERRESERVED) {
 /*	(osi_Msg "%s*WARNING* port number %d is not a reserved port number.  Use port numbers above %d\n", name, port, IPPORT_USERRESERVED);
 */ ;
@@ -124,10 +111,14 @@ rxi_GetHostUDPSocket(u_int ahost, u_short port)
 	goto error;
     }
 #endif
-    socketFd = socket(AF_INET, SOCK_DGRAM, 0);
+    socketFd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
 
-    if (socketFd < 0) {
+    if (socketFd == OSI_NULLSOCKET) {
+#ifdef AFS_NT40_ENV
+        fprintf(stderr, "socket() failed with error %u\n", WSAGetLastError());
+#else
 	perror("socket");
+#endif
 	goto error;
     }
 
@@ -146,17 +137,15 @@ rxi_GetHostUDPSocket(u_int ahost, u_short port)
 	if (binds)
 	    rxi_Delay(10);
 	code = bind(socketFd, (struct sockaddr *)&taddr, sizeof(taddr));
-	if (!code)
-	    break;
+        break;
     }
     if (code) {
-	perror("bind");
 	(osi_Msg "%sbind failed\n", name);
 	goto error;
     }
-#if !defined(AFS_NT40_ENV) 
+#if !defined(AFS_NT40_ENV)
     /*
-     * Set close-on-exec on rx socket 
+     * Set close-on-exec on rx socket
      */
     fcntl(socketFd, F_SETFD, 1);
 #endif
@@ -181,7 +170,7 @@ rxi_GetHostUDPSocket(u_int ahost, u_short port)
                 len2 /= 2;
         }
 
-        /* but do not let it get smaller than 32K */ 
+        /* but do not let it get smaller than 32K */
         if (len2 < len1)
             len2 = len1;
 
@@ -200,11 +189,8 @@ rxi_GetHostUDPSocket(u_int ahost, u_short port)
 	if (!greedy)
 	    (osi_Msg "%s*WARNING* Unable to increase buffering on socket\n",
 	     name);
-        if (rx_stats_active) {
-            MUTEX_ENTER(&rx_stats_mutex);
-            rx_stats.socketGreedy = greedy;
-            MUTEX_EXIT(&rx_stats_mutex);
-        }
+        if (rx_stats_active)
+            rx_atomic_set(&rx_stats.socketGreedy, greedy);
     }
 
 #ifdef AFS_LINUX22_ENV
@@ -221,10 +207,10 @@ rxi_GetHostUDPSocket(u_int ahost, u_short port)
 
   error:
 #ifdef AFS_NT40_ENV
-    if (socketFd >= 0)
+    if (socketFd != OSI_NULLSOCKET)
 	closesocket(socketFd);
 #else
-    if (socketFd >= 0)
+    if (socketFd != OSI_NULLSOCKET)
 	close(socketFd);
 #endif
 
@@ -247,7 +233,7 @@ osi_Panic(char *msg, ...)
     va_end(ap);
     fflush(stderr);
     fflush(stdout);
-    afs_abort();
+    opr_abort();
 }
 
 /*
@@ -267,9 +253,9 @@ static const char memZero;
 void *
 osi_Alloc(afs_int32 x)
 {
-    /* 
+    /*
      * 0-length allocs may return NULL ptr from malloc, so we special-case
-     * things so that NULL returned iff an error occurred 
+     * things so that NULL returned iff an error occurred
      */
     if (x == 0)
 	return (void *)&memZero;
@@ -296,7 +282,7 @@ static int myNetFlags[ADDRSPERSITE];
 static u_int rxi_numNetAddrs;
 static int Inited = 0;
 
-#if defined(AFS_NT40_ENV) 
+#if defined(AFS_NT40_ENV)
 int
 rxi_getaddr(void)
 {
@@ -306,15 +292,15 @@ rxi_getaddr(void)
     /* we don't want to use the loopback adapter which is first */
     /* this is a bad bad hack */
     if (rxi_numNetAddrs > 1)
-	return htonl(rxi_NetAddrs[1]);  
+	return htonl(rxi_NetAddrs[1]);
     else if (rxi_numNetAddrs > 0)
 	return htonl(rxi_NetAddrs[0]);
     else
 	return 0;
 }
 
-/* 
-** return number of addresses 
+/*
+** return number of addresses
 ** and the addresses themselves in the buffer
 ** maxSize - max number of interfaces to return.
 */
@@ -326,15 +312,6 @@ rx_getAllAddr(afs_uint32 * buffer, int maxSize)
     /* The IP address list can change so we must query for it */
     rx_GetIFInfo();
 
-#ifdef AFS_DJGPP_ENV
-    /* we don't want to use the loopback adapter which is first */
-    /* this is a bad bad hack.
-     * and doesn't hold true on Windows.
-     */
-    if ( rxi_numNetAddrs > 1 )
-        offset = 1;
-#endif /* AFS_DJGPP_ENV */
-
     for (count = 0; offset < rxi_numNetAddrs && maxSize > 0;
 	 count++, offset++, maxSize--)
 	buffer[count] = htonl(rxi_NetAddrs[offset]);
@@ -356,16 +333,7 @@ rx_getAllAddrMaskMtu(afs_uint32 addrBuffer[], afs_uint32 maskBuffer[],
     /* The IP address list can change so we must query for it */
     rx_GetIFInfo();
 
-#ifdef AFS_DJGPP_ENV
-    /* we don't want to use the loopback adapter which is first */
-    /* this is a bad bad hack.
-     * and doesn't hold true on Windows.
-     */
-    if ( rxi_numNetAddrs > 1 )
-        offset = 1;
-#endif /* AFS_DJGPP_ENV */
-
-    for (count = 0; 
+    for (count = 0;
          offset < rxi_numNetAddrs && maxSize > 0;
          count++, offset++, maxSize--) {
 	addrBuffer[count] = htonl(rxi_NetAddrs[offset]);
@@ -378,7 +346,7 @@ rx_getAllAddrMaskMtu(afs_uint32 addrBuffer[], afs_uint32 maskBuffer[],
 
 #ifdef AFS_NT40_ENV
 extern int rxinit_status;
-void 
+void
 rxi_InitMorePackets(void) {
     int npackets, ncbufs;
 
@@ -420,12 +388,14 @@ rx_GetIFInfo(void)
         maxsize =
             rxi_nRecvFrags * rxsize + (rxi_nRecvFrags - 1) * UDP_HDR_SIZE;
         maxsize = rxi_AdjustMaxMTU(rxsize, maxsize);
-        if (rx_maxReceiveSize < maxsize) {
+        if (rx_maxReceiveSize > maxsize) {
             rx_maxReceiveSize = MIN(RX_MAX_PACKET_SIZE, maxsize);
             rx_maxReceiveSize =
                 MIN(rx_maxReceiveSize, rx_maxReceiveSizeUser);
         }
-
+        if (rx_MyMaxSendSize > maxsize) {
+            rx_MyMaxSendSize = MIN(RX_MAX_PACKET_SIZE, maxsize);
+        }
     }
     UNLOCK_IF;
 
@@ -460,7 +430,7 @@ fudge_netmask(afs_uint32 addr)
 
 
 
-#if !defined(AFS_AIX_ENV) && !defined(AFS_NT40_ENV) && !defined(AFS_LINUX20_ENV) 
+#if !defined(AFS_AIX_ENV) && !defined(AFS_NT40_ENV) && !defined(AFS_LINUX20_ENV)
 int
 rxi_syscall(afs_uint32 a3, afs_uint32 a4, void *a5)
 {
@@ -470,7 +440,7 @@ rxi_syscall(afs_uint32 a3, afs_uint32 a4, void *a5)
     old = signal(SIGSYS, SIG_IGN);
 
 #if defined(AFS_SGI_ENV)
-    rcode = afs_syscall(a3, a4, a5);
+    rcode = afs_syscall(AFS_SYSCALL, 28, a3, a4, a5);
 #else
     rcode = syscall(AFS_SYSCALL, 28 /* AFSCALL_CALL */ , a3, a4, a5);
 #endif /* AFS_SGI_ENV */
@@ -509,10 +479,9 @@ rx_GetIFInfo(void)
     memset(myNetMTUs, 0, sizeof(myNetMTUs));
     memset(myNetMasks, 0, sizeof(myNetMasks));
     UNLOCK_IF;
-    s = socket(AF_INET, SOCK_DGRAM, 0);
-    if (s < 0)
+    s = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+    if (s == OSI_NULLSOCKET)
 	return;
-
 #ifdef	AFS_AIX41_ENV
     ifc.ifc_len = sizeof(buf);
     ifc.ifc_buf = buf;
@@ -557,7 +526,7 @@ rx_GetIFInfo(void)
 	if (a->sin_family != AF_INET)
 	    continue;
 	rxi_NetAddrs[rxi_numNetAddrs] = ntohl(a->sin_addr.s_addr);
-	if (rxi_NetAddrs[rxi_numNetAddrs] == 0x7f000001) {
+	if (rx_IsLoopbackAddr(rxi_NetAddrs[rxi_numNetAddrs])) {
 	    /* we don't really care about "localhost" */
 	    continue;
 	}
@@ -580,7 +549,7 @@ rx_GetIFInfo(void)
 	    if (ifr->ifr_flags & IFF_LOOPBACK)
 		continue;
 #endif
-	    /* fprintf(stderr, "if %s flags=%x\n", 
+	    /* fprintf(stderr, "if %s flags=%x\n",
 	     * ifr->ifr_name, ifr->ifr_flags); */
 	} else {		/*
 				 * fputs(stderr, "ioctl error IFFLAGS\n");
@@ -593,8 +562,8 @@ rx_GetIFInfo(void)
 	rxi_syscallp = rxi_syscall;
 #endif
 
-	/* If I refer to kernel extensions that aren't loaded on AIX, the 
-	 * program refuses to load and run, so I simply can't include the 
+	/* If I refer to kernel extensions that aren't loaded on AIX, the
+	 * program refuses to load and run, so I simply can't include the
 	 * following code.  Fortunately, AIX is the one operating system in
 	 * which the subsequent ioctl works reliably. */
 	if (rxi_syscallp) {
@@ -614,7 +583,7 @@ rx_GetIFInfo(void)
 	    } else
 		myNetMasks[rxi_numNetAddrs] =
 		    ntohl(myNetMasks[rxi_numNetAddrs]);
-	    /* fprintf(stderr, "if %s mask=0x%x\n", 
+	    /* fprintf(stderr, "if %s mask=0x%x\n",
 	     * ifr->ifr_name, myNetMasks[rxi_numNetAddrs]); */
 	}
 
@@ -624,7 +593,7 @@ rx_GetIFInfo(void)
 	    res = ioctl(s, SIOCGIFMTU, ifr);
 	    if ((res == 0) && (ifr->ifr_metric > 128)) {	/* sanity check */
 		myNetMTUs[rxi_numNetAddrs] = ifr->ifr_metric;
-		/* fprintf(stderr, "if %s mtu=%d\n", 
+		/* fprintf(stderr, "if %s mtu=%d\n",
 		 * ifr->ifr_name, ifr->ifr_metric); */
 	    } else {
 		/* fputs(stderr, "ioctl error IFMTU\n");
@@ -641,7 +610,7 @@ rx_GetIFInfo(void)
 	    if ((res == 0)) {
 		a = (struct sockaddr_in *)&ifr->ifr_addr;
 		myNetMasks[rxi_numNetAddrs] = ntohl(a->sin_addr.s_addr);
-		/* fprintf(stderr, "if %s subnetmask=0x%x\n", 
+		/* fprintf(stderr, "if %s subnetmask=0x%x\n",
 		 * ifr->ifr_name, myNetMasks[rxi_numNetAddrs]); */
 	    } else {
 		/* fputs(stderr, "ioctl error IFMASK\n");
@@ -650,7 +619,7 @@ rx_GetIFInfo(void)
 #endif
 	}
 
-	if (rxi_NetAddrs[rxi_numNetAddrs] != 0x7f000001) {	/* ignore lo0 */
+	if (!rx_IsLoopbackAddr(rxi_NetAddrs[rxi_numNetAddrs])) {	/* ignore lo0 */
 	    int maxsize;
 	    maxsize =
 		rxi_nRecvFrags * (myNetMTUs[rxi_numNetAddrs] - RX_IP_SIZE);
@@ -701,8 +670,6 @@ rxi_InitPeerParams(struct rx_peer *pp)
     struct sockaddr_in addr;
 #endif
 
-
-
     LOCK_IF_INIT;
     if (!Inited) {
 	UNLOCK_IF_INIT;
@@ -721,19 +688,19 @@ rxi_InitPeerParams(struct rx_peer *pp)
     ppaddr = ntohl(pp->host);
 
     pp->ifMTU = 0;
-    pp->timeout.sec = 2;
-    pp->rateFlag = 2;		/* start timing after two full packets */
-    /* I don't initialize these, because I presume they are bzero'd... 
+    rx_rto_setPeerTimeoutSecs(pp, 2);
+    /* I don't initialize these, because I presume they are bzero'd...
      * pp->burstSize pp->burst pp->burstWait.sec pp->burstWait.usec
-     * pp->timeout.usec */
+     */
 
     LOCK_IF;
     for (ix = 0; ix < rxi_numNetAddrs; ++ix) {
 	if ((rxi_NetAddrs[ix] & myNetMasks[ix]) == (ppaddr & myNetMasks[ix])) {
 #ifdef IFF_POINTOPOINT
 	    if (myNetFlags[ix] & IFF_POINTOPOINT)
-		pp->timeout.sec = 4;
+		rx_rto_setPeerTimeoutSecs(pp, 4);
 #endif /* IFF_POINTOPOINT */
+
 	    rxmtu = myNetMTUs[ix] - RX_IPUDP_SIZE;
 	    if (rxmtu < RX_MIN_PACKET_SIZE)
 		rxmtu = RX_MIN_PACKET_SIZE;
@@ -743,28 +710,31 @@ rxi_InitPeerParams(struct rx_peer *pp)
     }
     UNLOCK_IF;
     if (!pp->ifMTU) {		/* not local */
-	pp->timeout.sec = 3;
+	rx_rto_setPeerTimeoutSecs(pp, 3);
 	pp->ifMTU = MIN(rx_MyMaxSendSize, RX_REMOTE_PACKET_SIZE);
     }
 #else /* ADAPT_MTU */
-    pp->rateFlag = 2;		/* start timing after two full packets */
-    pp->timeout.sec = 2;
+    rx_rto_setPeerTimeoutSecs(pp, 2);
     pp->ifMTU = MIN(rx_MyMaxSendSize, OLD_MAX_PACKET_SIZE);
 #endif /* ADAPT_MTU */
 #if defined(ADAPT_PMTU) && defined(IP_MTU)
     sock=socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
-    if (sock >= 0) {
-      addr.sin_family = AF_INET;
-      addr.sin_addr.s_addr = pp->host;
-      addr.sin_port = pp->port;
-      if (connect(sock, (struct sockaddr *)&addr, sizeof(addr)) == 0) {
-	int mtu=0;
-        socklen_t s = sizeof(mtu);
-	if (getsockopt(sock, SOL_IP, IP_MTU, &mtu, &s)== 0) {
-	  pp->ifMTU = MIN(mtu - RX_IPUDP_SIZE, pp->ifMTU);
-	}
-      }
-      close(sock);
+    if (sock != OSI_NULLSOCKET) {
+        addr.sin_family = AF_INET;
+        addr.sin_addr.s_addr = pp->host;
+        addr.sin_port = pp->port;
+        if (connect(sock, (struct sockaddr *)&addr, sizeof(addr)) == 0) {
+            int mtu=0;
+            socklen_t s = sizeof(mtu);
+            if (getsockopt(sock, SOL_IP, IP_MTU, &mtu, &s)== 0) {
+                pp->ifMTU = MIN(mtu - RX_IPUDP_SIZE, pp->ifMTU);
+            }
+        }
+#ifdef AFS_NT40_ENV
+        closesocket(sock);
+#else
+        close(sock);
+#endif
     }
 #endif
     pp->ifMTU = rxi_AdjustIfMTU(pp->ifMTU);
@@ -792,7 +762,7 @@ rx_SetNoJumbo(void)
     rxi_nSendFrags = rxi_nRecvFrags = 1;
 }
 
-/* Override max MTU.  If rx_SetNoJumbo is called, it must be 
+/* Override max MTU.  If rx_SetNoJumbo is called, it must be
    called before calling rx_SetMaxMTU since SetNoJumbo clobbers rx_maxReceiveSize */
 void
 rx_SetMaxMTU(int mtu)
@@ -800,17 +770,17 @@ rx_SetMaxMTU(int mtu)
     rx_MyMaxSendSize = rx_maxReceiveSizeUser = rx_maxReceiveSize = mtu;
 }
 
-#if defined(HAVE_LINUX_ERRQUEUE_H) && defined(ADAPT_PMTU)
+#if defined(ADAPT_PMTU)
 int
 rxi_HandleSocketError(int socket)
 {
+    int ret=0;
+#if defined(HAVE_LINUX_ERRQUEUE_H)
     struct msghdr msg;
     struct cmsghdr *cmsg;
     struct sock_extended_err *err;
     struct sockaddr_in addr;
-    struct sockaddr *offender;
     char controlmsgbuf[256];
-    int ret=0;
     int code;
 
     msg.msg_name = &addr;
@@ -839,14 +809,15 @@ rxi_HandleSocketError(int socket)
         goto out;
     ret=1;
     err =(struct sock_extended_err *) CMSG_DATA(cmsg);
-    
+
     if (err->ee_errno == EMSGSIZE && err->ee_info >= 68) {
-        rxi_SetPeerMtu(addr.sin_addr.s_addr, addr.sin_port,
+	rxi_SetPeerMtu(NULL, addr.sin_addr.s_addr, addr.sin_port,
                        err->ee_info - RX_IPUDP_SIZE);
     }
     /* other DEST_UNREACH's and TIME_EXCEEDED should be dealt with too */
-    
+
 out:
+#endif
     return ret;
 }
 #endif
