@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -17,15 +17,16 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_xmit_nt.c,v 1.11.2.1 2009/01/27 06:05:04 jaltman Exp $");
-
 #if defined(AFS_NT40_ENV)
+# include <roken.h>
+# include <winsock2.h>
+# if (_WIN32_WINNT < 0x0501)
+#  undef _WIN32_WINNT
+#  define _WIN32_WINNT 0x0501
+# endif
+# include <mswsock.h>
 
-#include <winsock2.h>
-#include <mswsock.h>
-
-#if (_WIN32_WINNT < 0x0600)
+# if (_WIN32_WINNT < 0x0600)
 /*
  * WSASendMsg -- send data to a specific destination, with options, using
  *    overlapped I/O where applicable.
@@ -49,9 +50,9 @@ INT
     IN LPWSAOVERLAPPED_COMPLETION_ROUTINE lpCompletionRoutine OPTIONAL
     );
 
-#define WSAID_WSASENDMSG /* a441e712-754f-43ca-84a7-0dee44cf606d */ \
+# define WSAID_WSASENDMSG /* a441e712-754f-43ca-84a7-0dee44cf606d */ \
     {0xa441e712,0x754f,0x43ca,{0x84,0xa7,0x0d,0xee,0x44,0xcf,0x60,0x6d}}
-#endif /* _WINNT_WIN32 */
+#endif /* AFS_NT40_ENV */
 
 #include "rx.h"
 #include "rx_packet.h"
@@ -61,10 +62,10 @@ INT
 #include <errno.h>
 
 
-/* 
+/*
  * WSASendMsg is only supported on Vista and above
  * Neither function is part of the public WinSock API
- * and therefore the function pointers must be 
+ * and therefore the function pointers must be
  * obtained via WSAIoctl()
  */
 static LPFN_WSARECVMSG pWSARecvMsg = NULL;
@@ -81,19 +82,26 @@ rxi_xmit_init(osi_socket s)
     rc = WSAIoctl( s, SIO_GET_EXTENSION_FUNCTION_POINTER,
                    &WSARecvMsg_GUID, sizeof(WSARecvMsg_GUID),
                    &pWSARecvMsg, sizeof(pWSARecvMsg),
-                   &NumberOfBytes, NULL, NULL); 
+                   &NumberOfBytes, NULL, NULL);
 
     rc = WSAIoctl( s, SIO_GET_EXTENSION_FUNCTION_POINTER,
                    &WSASendMsg_GUID, sizeof(WSASendMsg_GUID),
                    &pWSASendMsg, sizeof(pWSASendMsg),
-                   &NumberOfBytes, NULL, NULL); 
+                   &NumberOfBytes, NULL, NULL);
 
     /* Turn on UDP PORT_UNREACHABLE messages */
     dwIn = 1;
-    rc = WSAIoctl( s, SIO_UDP_CONNRESET, 
+    rc = WSAIoctl( s, SIO_UDP_CONNRESET,
                    &dwIn, sizeof(dwIn),
                    &dwOut, sizeof(dwOut),
-                   &NumberOfBytes, NULL, NULL); 
+                   &NumberOfBytes, NULL, NULL);
+
+    /* Turn on UDP CIRCULAR QUEUEING messages */
+    dwIn = 1;
+    rc = WSAIoctl( s, SIO_ENABLE_CIRCULAR_QUEUEING,
+                   &dwIn, sizeof(dwIn),
+                   &dwOut, sizeof(dwOut),
+                   &NumberOfBytes, NULL, NULL);
 }
 
 int
@@ -115,7 +123,7 @@ recvmsg(osi_socket socket, struct msghdr *msgP, int flags)
         wsaMsg.dwFlags = flags;
 
         code = pWSARecvMsg(socket, &wsaMsg, &dwBytes, NULL, NULL);
-        if (code == 0) { 
+        if (code == 0) {
             /* success - return the number of bytes read */
             code = (int)dwBytes;
         } else {
@@ -190,7 +198,7 @@ sendmsg(osi_socket socket, struct msghdr *msgP, int flags)
         wsaMsg.dwFlags = 0;
 
         code = pWSASendMsg(socket, &wsaMsg, flags, &dwBytes, NULL, NULL);
-        if (code == 0) { 
+        if (code == 0) {
             /* success - return the number of bytes read */
             code = (int)dwBytes;
         } else {
