@@ -9,39 +9,23 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
 #include <afs/stds.h>
-#include <sys/types.h>
-#include <signal.h>
-#include <sys/stat.h>
-#include <string.h>
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
+
+#include <roken.h>
+
 #ifdef AFS_NT40_ENV
-#include <winsock2.h>
 #include <WINNT/afsevent.h>
 #endif
+
 #ifdef HAVE_SYS_FILE_H
 #include <sys/file.h>
 #endif
-#include <time.h>
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
-#ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
-#endif
-#include <stdio.h>
-#ifdef HAVE_STDINT_H
-# include <stdint.h>
-#endif
 
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <rx/rx_globals.h>
 #include <rx/rxstat.h>
+#include <afs/cmd.h>
 #include <afs/cellconfig.h>
 #include <afs/keys.h>
 #include <afs/auth.h>
@@ -50,17 +34,18 @@
 #include <lock.h>
 #include <ubik.h>
 #include <afs/afsutil.h>
+
 #include "vlserver.h"
 #include "vlserver_internal.h"
 
 #define MAXLWP 16
-const char *vl_dbaseName;
 struct afsconf_dir *vldb_confdir = 0;	/* vldb configuration dir */
 int lwps = 9;
 
 struct vldstats dynamic_statistics;
 struct ubik_dbase *VL_dbase;
-afs_uint32 HostAddress[MAXSERVERID + 1];
+afs_uint32 rd_HostAddress[MAXSERVERID + 1];
+afs_uint32 wr_HostAddress[MAXSERVERID + 1];
 
 static void *CheckSignal(void*);
 int LogLevel = 0;
@@ -69,7 +54,6 @@ int rxJumbograms = 0;		/* default is to not send and receive jumbo grams */
 int rxMaxMTU = -1;
 afs_int32 rxBind = 0;
 int rxkadDisableDotCheck = 0;
-int debuglevel = 0;
 
 #define ADDRSPERSITE 16         /* Same global is in rx/rx_user.c */
 afs_uint32 SHostAddrs[ADDRSPERSITE];
@@ -88,20 +72,20 @@ static void *
 CheckSignal(void *unused)
 {
     int i, errorcode;
-    struct ubik_trans *trans;
+    struct vl_ctx ctx;
 
     if ((errorcode =
-	Init_VLdbase(&trans, LOCKREAD, VLGETSTATS - VL_LOWEST_OPCODE)))
+	Init_VLdbase(&ctx, LOCKREAD, VLGETSTATS - VL_LOWEST_OPCODE)))
 	return (void *)(intptr_t)errorcode;
     VLog(0, ("Dump name hash table out\n"));
     for (i = 0; i < HASHSIZE; i++) {
-	HashNDump(trans, i);
+	HashNDump(&ctx, i);
     }
     VLog(0, ("Dump id hash table out\n"));
     for (i = 0; i < HASHSIZE; i++) {
-	HashIdDump(trans, i);
+	HashIdDump(&ctx, i);
     }
-    return ((void *)(intptr_t)ubik_EndTrans(trans));
+    return ((void *)(intptr_t)ubik_EndTrans(ctx.trans));
 }				/*CheckSignal */
 
 
@@ -129,6 +113,27 @@ vldb_rxstat_userok(struct rx_call *call)
 
 #include "AFS_component_version_number.c"
 
+enum optionsList {
+    OPT_noauth,
+    OPT_smallmem,
+    OPT_auditlog,
+    OPT_auditiface,
+    OPT_config,
+    OPT_debug,
+    OPT_database,
+    OPT_logfile,
+    OPT_threads,
+    OPT_syslog,
+    OPT_peer,
+    OPT_process,
+    OPT_nojumbo,
+    OPT_jumbo,
+    OPT_rxbind,
+    OPT_rxmaxmtu,
+    OPT_trace,
+    OPT_dotted
+};
+
 int
 main(int argc, char **argv)
 {
@@ -142,10 +147,18 @@ main(int argc, char **argv)
     struct afsconf_cell info;
     struct hostent *th;
     char hostname[VL_MAXNAMELEN];
-    int noAuth = 0, index;
+    int noAuth = 0;
     char clones[MAXHOSTSPERCELL];
-    char *auditFileName = NULL;
     afs_uint32 host = ntohl(INADDR_ANY);
+    struct cmd_syndesc *opts;
+
+    char *vl_dbaseName;
+    char *configDir;
+    char *logFile;
+
+    char *auditFileName = NULL;
+    char *interface = NULL;
+    char *optstring = NULL;
 
 #ifdef	AFS_AIX32_ENV
     /*
@@ -165,101 +178,6 @@ main(int argc, char **argv)
 #endif
     osi_audit_init();
 
-    /* Parse command line */
-    for (index = 1; index < argc; index++) {
-	if (strcmp(argv[index], "-noauth") == 0) {
-	    noAuth = 1;
-	} else if (strcmp(argv[index], "-p") == 0) {
-	    lwps = atoi(argv[++index]);
-	    if (lwps > MAXLWP) {
-		printf("Warning: '-p %d' is too big; using %d instead\n",
-		       lwps, MAXLWP);
-		lwps = MAXLWP;
-	    }
-	} else if (strcmp(argv[index], "-d") == 0) {
-	    if ((index + 1) >= argc) {
-		fprintf(stderr, "missing argument for -d\n");
-		return -1;
-	    }
-	    debuglevel = atoi(argv[++index]);
-	    LogLevel = debuglevel;
-	} else if (strcmp(argv[index], "-nojumbo") == 0) {
-	    rxJumbograms = 0;
-	} else if (strcmp(argv[index], "-jumbo") == 0) {
-	    rxJumbograms = 1;
-	} else if (strcmp(argv[index], "-rxbind") == 0) {
-	    rxBind = 1;
-	} else if (strcmp(argv[index], "-allow-dotted-principals") == 0) {
-	    rxkadDisableDotCheck = 1;
-	} else if (!strcmp(argv[index], "-rxmaxmtu")) {
-	    if ((index + 1) >= argc) {
-		fprintf(stderr, "missing argument for -rxmaxmtu\n");
-		return -1;
-	    }
-	    rxMaxMTU = atoi(argv[++index]);
-	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) ||
-		(rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
-		printf("rxMaxMTU %d invalid; must be between %d-%" AFS_SIZET_FMT "\n",
-		       rxMaxMTU, RX_MIN_PACKET_SIZE,
-		       RX_MAX_PACKET_DATA_SIZE);
-		return -1;
-	    }
-
-	} else if (strcmp(argv[index], "-smallmem") == 0) {
-	    smallMem = 1;
-
-	} else if (strcmp(argv[index], "-trace") == 0) {
-	    extern char rxi_tracename[80];
-	    strcpy(rxi_tracename, argv[++index]);
-
-	} else if (strcmp(argv[index], "-auditlog") == 0) {
-	    auditFileName = argv[++index];
-
-	} else if (strcmp(argv[index], "-audit-interface") == 0) {
-	    char *interface = argv[++index];
-
-	    if (osi_audit_interface(interface)) {
-		printf("Invalid audit interface '%s'\n", interface);
-		return -1;
-	    }
-
-	} else if (strcmp(argv[index], "-enable_peer_stats") == 0) {
-	    rx_enablePeerRPCStats();
-	} else if (strcmp(argv[index], "-enable_process_stats") == 0) {
-	    rx_enableProcessRPCStats();
-#ifndef AFS_NT40_ENV
-	} else if (strcmp(argv[index], "-syslog") == 0) {
-	    /* set syslog logging flag */
-	    serverLogSyslog = 1;
-	} else if (strncmp(argv[index], "-syslog=", 8) == 0) {
-	    serverLogSyslog = 1;
-	    serverLogSyslogFacility = atoi(argv[index] + 8);
-#endif
-	} else {
-	    /* support help flag */
-#ifndef AFS_NT40_ENV
-	    printf("Usage: vlserver [-p <number of processes>] [-nojumbo] "
-		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals] "
-		   "[-auditlog <log path>] [-jumbo] [-d <debug level>] "
-		   "[-syslog[=FACILITY]] "
-		   "[-enable_peer_stats] [-enable_process_stats] "
-		   "[-help]\n");
-#else
-	    printf("Usage: vlserver [-p <number of processes>] [-nojumbo] "
-		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals] "
-		   "[-auditlog <log path>] [-jumbo] [-d <debug level>] "
-		   "[-enable_peer_stats] [-enable_process_stats] "
-		   "[-help]\n");
-#endif
-	    fflush(stdout);
-	    exit(0);
-	}
-    }
-
-    if (auditFileName) {
-	osi_audit_file(auditFileName);
-    }
-
     /* Initialize dirpaths */
     if (!(initAFSDirPath() & AFSDIR_SERVER_PATHS_OK)) {
 #ifdef AFS_NT40_ENV
@@ -269,26 +187,155 @@ main(int argc, char **argv)
 		argv[0]);
 	exit(2);
     }
-    vl_dbaseName = AFSDIR_SERVER_VLDB_FILEPATH;
+
+    vl_dbaseName = strdup(AFSDIR_SERVER_VLDB_FILEPATH);
+    configDir = strdup(AFSDIR_SERVER_ETC_DIRPATH);
+    logFile = strdup(AFSDIR_SERVER_VLOG_FILEPATH);
+
+    cmd_DisableAbbreviations();
+    cmd_DisablePositionalCommands();
+    opts = cmd_CreateSyntax(NULL, NULL, NULL, NULL);
+
+    /* vlserver specific options */
+    cmd_AddParmAtOffset(opts, OPT_noauth, "-noauth", CMD_FLAG,
+		        CMD_OPTIONAL, "disable authentication");
+    cmd_AddParmAtOffset(opts, OPT_smallmem, "-smallmem", CMD_FLAG,
+		        CMD_OPTIONAL, "optimise for small memory systems");
+
+    /* general server options */
+    cmd_AddParmAtOffset(opts, OPT_auditlog, "-auditlog", CMD_SINGLE,
+		        CMD_OPTIONAL, "location of audit log");
+    cmd_AddParmAtOffset(opts, OPT_auditiface, "-audit-interface", CMD_SINGLE,
+		        CMD_OPTIONAL, "interface to use for audit logging");
+    cmd_AddParmAtOffset(opts, OPT_config, "-config", CMD_SINGLE,
+		        CMD_OPTIONAL, "configuration location");
+    cmd_AddParmAtOffset(opts, OPT_debug, "-d", CMD_SINGLE,
+		        CMD_OPTIONAL, "debug level");
+    cmd_AddParmAtOffset(opts, OPT_database, "-database", CMD_SINGLE,
+		        CMD_OPTIONAL, "database file");
+    cmd_AddParmAlias(opts, OPT_database, "-db");
+    cmd_AddParmAtOffset(opts, OPT_logfile, "-logfile", CMD_SINGLE,
+		        CMD_OPTIONAL, "location of logfile");
+    cmd_AddParmAtOffset(opts, OPT_threads, "-p", CMD_SINGLE, CMD_OPTIONAL,
+		        "number of threads");
+#if !defined(AFS_NT40_ENV)
+    cmd_AddParmAtOffset(opts, OPT_syslog, "-syslog", CMD_SINGLE_OR_FLAG,
+		        CMD_OPTIONAL, "log to syslog");
+#endif
+
+    /* rx options */
+    cmd_AddParmAtOffset(opts, OPT_peer, "-enable_peer_stats", CMD_FLAG,
+		        CMD_OPTIONAL, "enable RX transport statistics");
+    cmd_AddParmAtOffset(opts, OPT_process, "-enable_process_stats", CMD_FLAG,
+		        CMD_OPTIONAL, "enable RX RPC statistics");
+    cmd_AddParmAtOffset(opts, OPT_nojumbo, "-nojumbo", CMD_FLAG,
+		        CMD_OPTIONAL, "disable jumbograms");
+    cmd_AddParmAtOffset(opts, OPT_jumbo, "-jumbo", CMD_FLAG,
+		        CMD_OPTIONAL, "enable jumbograms");
+    cmd_AddParmAtOffset(opts, OPT_rxbind, "-rxbind", CMD_FLAG,
+		        CMD_OPTIONAL, "bind only to the primary interface");
+    cmd_AddParmAtOffset(opts, OPT_rxmaxmtu, "-rxmaxmtu", CMD_SINGLE,
+		        CMD_OPTIONAL, "maximum MTU for RX");
+    cmd_AddParmAtOffset(opts, OPT_trace, "-trace", CMD_SINGLE,
+		        CMD_OPTIONAL, "rx trace file");
+
+    /* rxkad options */
+    cmd_AddParmAtOffset(opts, OPT_dotted, "-allow-dotted-principals",
+		        CMD_FLAG, CMD_OPTIONAL,
+		        "permit Kerberos 5 principals with dots");
+
+    code = cmd_Parse(argc, argv, &opts);
+    if (code)
+	return -1;
+
+    /* vlserver options */
+    cmd_OptionAsFlag(opts, OPT_noauth, &noAuth);
+    cmd_OptionAsFlag(opts, OPT_smallmem, &smallMem);
+    if (cmd_OptionAsString(opts, OPT_trace, &optstring) == 0) {
+	extern char rxi_tracename[80];
+	strcpy(rxi_tracename, optstring);
+	free(optstring);
+	optstring = NULL;
+    }
+
+    /* general server options */
+
+    cmd_OptionAsString(opts, OPT_auditlog, &auditFileName);
+
+    if (cmd_OptionAsString(opts, OPT_auditiface, &interface) == 0) {
+	if (osi_audit_interface(interface)) {
+	    printf("Invalid audit interface '%s'\n", interface);
+	    return -1;
+	}
+	free(interface);
+    }
+
+    cmd_OptionAsString(opts, OPT_config, &configDir);
+    cmd_OptionAsInt(opts, OPT_debug, &LogLevel);
+    cmd_OptionAsString(opts, OPT_database, &vl_dbaseName);
+    cmd_OptionAsString(opts, OPT_logfile, &logFile);
+
+    if (cmd_OptionAsInt(opts, OPT_threads, &lwps) == 0) {
+	if (lwps > MAXLWP) {
+	     printf("Warning: '-p %d' is too big; using %d instead\n",
+		    lwps, MAXLWP);
+	     lwps = MAXLWP;
+	}
+    }
+#ifndef AFS_NT40_ENV
+    if (cmd_OptionPresent(opts, OPT_syslog)) {
+        serverLogSyslog = 1;
+        cmd_OptionAsInt(opts, OPT_syslog, &serverLogSyslogFacility);
+    }
+#endif
+
+    /* rx options */
+    if (cmd_OptionPresent(opts, OPT_peer))
+	rx_enablePeerRPCStats();
+    if (cmd_OptionPresent(opts, OPT_process))
+	rx_enableProcessRPCStats();
+    if (cmd_OptionPresent(opts, OPT_nojumbo))
+	rxJumbograms = 0;
+    if (cmd_OptionPresent(opts, OPT_jumbo))
+	rxJumbograms = 1;
+
+    cmd_OptionAsFlag(opts, OPT_rxbind, &rxBind);
+
+    if (cmd_OptionAsInt(opts, OPT_rxmaxmtu, &rxMaxMTU) == 0) {
+	if ((rxMaxMTU < RX_MIN_PACKET_SIZE) ||
+	    (rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
+	    printf("rxMaxMTU %d invalid; must be between %d-%" AFS_SIZET_FMT "\n",
+		   rxMaxMTU, RX_MIN_PACKET_SIZE,
+		   RX_MAX_PACKET_DATA_SIZE);
+	    return -1;
+	}
+    }
+
+    /* rxkad options */
+    cmd_OptionAsFlag(opts, OPT_dotted, &rxkadDisableDotCheck);
+
+    if (auditFileName) {
+	osi_audit_file(auditFileName);
+    }
 
 #ifndef AFS_NT40_ENV
     serverLogSyslogTag = "vlserver";
 #endif
-    OpenLog(AFSDIR_SERVER_VLOG_FILEPATH);	/* set up logging */
+    OpenLog(logFile);	/* set up logging */
     SetupLogSignals();
 
-    tdir = afsconf_Open(AFSDIR_SERVER_ETC_DIRPATH);
+    tdir = afsconf_Open(configDir);
     if (!tdir) {
-	VLog(0,
+	printf
 	    ("vlserver: can't open configuration files in dir %s, giving up.\n",
-	     AFSDIR_SERVER_ETC_DIRPATH));
+	     configDir);
 	exit(1);
     }
 #ifdef AFS_NT40_ENV
     /* initialize winsock */
     if (afs_winsockInit() < 0) {
 	ReportErrorEventAlt(AFSEVT_SVR_WINSOCK_INIT_FAILED, 0, argv[0], 0);
-	VLog(0, ("vlserver: couldn't initialize winsock.\n"));
+	fprintf(stderr, "vlserver: couldn't initialize winsock. \n");
 	exit(1);
     }
 #endif
@@ -296,8 +343,8 @@ main(int argc, char **argv)
     gethostname(hostname, sizeof(hostname));
     th = gethostbyname(hostname);
     if (!th) {
-	VLog(0, ("vlserver: couldn't get address of this host (%s).\n",
-	       hostname));
+	printf("vlserver: couldn't get address of this host (%s).\n",
+	       hostname);
 	exit(1);
     }
     memcpy(&myHost, th->h_addr, sizeof(afs_uint32));
@@ -310,7 +357,7 @@ main(int argc, char **argv)
 	afsconf_GetExtendedCellInfo(tdir, NULL, AFSCONF_VLDBSERVICE, &info,
 				    clones);
     if (code) {
-	VLog(0, ("vlserver: Couldn't get cell server list for 'afsvldb'.\n"));
+	printf("vlserver: Couldn't get cell server list for 'afsvldb'.\n");
 	exit(2);
     }
 
@@ -343,12 +390,11 @@ main(int argc, char **argv)
     }
 
     ubik_nBuffers = 512;
-    ubik_CRXSecurityProc = afsconf_ClientAuth;
-    ubik_CRXSecurityRock = (char *)tdir;
-    ubik_SRXSecurityProc = afsconf_ServerAuth;
-    ubik_SRXSecurityRock = (char *)tdir;
-    ubik_CheckRXSecurityProc = afsconf_CheckAuth;
-    ubik_CheckRXSecurityRock = (char *)tdir;
+    ubik_SetClientSecurityProcs(afsconf_ClientAuth, afsconf_UpToDate, tdir);
+    ubik_SetServerSecurityProcs(afsconf_BuildServerSecurityObjects,
+				afsconf_CheckAuth, tdir);
+
+    ubik_SyncWriterCacheProc = vlsynccache;
     code =
 	ubik_ServerInitByInfo(myHost, htons(AFSCONF_VLDBPORT), &info, clones,
 			      vl_dbaseName, &VL_dbase);
@@ -364,10 +410,11 @@ main(int argc, char **argv)
     }
     rx_SetRxDeadTime(50);
 
-    memset(HostAddress, 0, sizeof(HostAddress));
+    memset(rd_HostAddress, 0, sizeof(rd_HostAddress));
+    memset(wr_HostAddress, 0, sizeof(wr_HostAddress));
     initialize_dstats();
 
-    afsconf_BuildServerSecurityObjects(tdir, 0, &securityClasses, &numClasses);
+    afsconf_BuildServerSecurityObjects(tdir, &securityClasses, &numClasses);
 
     tservice =
 	rx_NewServiceHost(host, 0, USER_SERVICE_ID, "Vldb server",
@@ -392,7 +439,7 @@ main(int argc, char **argv)
 			  securityClasses, numClasses,
 			  RXSTATS_ExecuteRequest);
     if (tservice == (struct rx_service *)0) {
-	VLog(0, ("vlserver: Could not create rpc stats rx service\n"));
+	printf("vlserver: Could not create rpc stats rx service\n");
 	exit(3);
     }
     rx_SetMinProcs(tservice, 2);
