@@ -18,7 +18,6 @@
 #include <rx/rx.h>
 #include <rx/rxkad.h>
 #include <afs/afsint.h>
-#include <afs/afs_assert.h>
 #include <afs/prs_fs.h>
 #include <afs/nfs.h>
 #include <lwp.h>
@@ -728,8 +727,6 @@ VolClone(struct rx_call *acid, afs_int32 atrans, afs_uint32 purgeId,
     }
     newId = *newNumber;
 
-    if (newType != readonlyVolume && newType != backupVolume)
-	return EINVAL;
     tt = FindTrans(atrans);
     if (!tt)
 	return ENOENT;
@@ -779,15 +776,8 @@ VolClone(struct rx_call *acid, afs_int32 atrans, afs_uint32 purgeId,
 	    error = EINVAL;
 	    goto fail;
 	}
-	if (V_type(originalvp) == readonlyVolume
-	    && V_parentId(originalvp) != V_parentId(purgevp)) {
-	    Log("1 Volser: Clone: Volume %u and volume %u were not cloned from the same parent volume; aborted\n", tt->volid, purgeId);
-	    error = EXDEV;
-	    goto fail;
-	}
-	if (V_type(originalvp) == readwriteVolume
-	    && tt->volid != V_parentId(purgevp)) {
-	    Log("1 Volser: Clone: Volume %u was not originally cloned from volume %u; aborted\n", purgeId, tt->volid);
+	if (V_parentId(originalvp) != V_parentId(purgevp)) {
+	    Log("1 Volser: Clone: Volume %u and volume %u were not originally cloned from the same parent; aborted\n", purgeId, tt->volid);
 	    error = EXDEV;
 	    goto fail;
 	}
@@ -931,12 +921,6 @@ VolReClone(struct rx_call *acid, afs_int32 atrans, afs_int32 cloneId)
     TSetRxCall(tt, acid, "ReClone");
 
     originalvp = tt->volume;
-    if ((V_type(originalvp) == backupVolume)
-	|| (V_type(originalvp) == readonlyVolume)) {
-	Log("1 Volser: Clone: The volume to be cloned must be a read/write; aborted\n");
-	error = EROFS;
-	goto fail;
-    }
     if ((V_destroyMe(originalvp) == DESTROY_ME) || !V_inService(originalvp)) {
 	Log("1 Volser: Clone: Volume %d is offline and cannot be cloned\n",
 	    V_id(originalvp));
@@ -958,19 +942,7 @@ VolReClone(struct rx_call *acid, afs_int32 atrans, afs_int32 cloneId)
 	error = EXDEV;
 	goto fail;
     }
-    if (V_type(clonevp) != readonlyVolume && V_type(clonevp) != backupVolume) {
-	Log("1 Volser: Clone: The \"recloned\" volume must be a read only volume; aborted\n");
-	error = EINVAL;
-	goto fail;
-    }
-    if (V_type(originalvp) == readonlyVolume
-	&& V_parentId(originalvp) != V_parentId(clonevp)) {
-	Log("1 Volser: Clone: Volume %u and volume %u were not cloned from the same parent volume; aborted\n", tt->volid, cloneId);
-	error = EXDEV;
-	goto fail;
-    }
-    if (V_type(originalvp) == readwriteVolume
-	&& tt->volid != V_parentId(clonevp)) {
+    if (V_parentId(originalvp) != V_parentId(clonevp)) {
 	Log("1 Volser: Clone: Volume %u was not originally cloned from volume %u; aborted\n", cloneId, tt->volid);
 	error = EXDEV;
 	goto fail;
@@ -1925,22 +1897,6 @@ XVolListPartitions(struct rx_call *acid, struct partEntries *pEntries)
 
 }
 
-/*extract the volume id from string vname. Its of the form " V0*<id>.vol  "*/
-afs_int32
-ExtractVolId(char vname[])
-{
-    int i;
-    char name[VOLSER_MAXVOLNAME + 1];
-
-    strcpy(name, vname);
-    i = 0;
-    while (name[i] == 'V' || name[i] == '0')
-	i++;
-
-    name[11] = '\0';		/* smash the "." */
-    return (atol(&name[i]));
-}
-
 /*return the name of the next volume header in the directory associated with dirp and dp.
 *the volume id is  returned in volid, and volume header name is returned in volname*/
 int
@@ -1951,7 +1907,7 @@ GetNextVol(DIR * dirp, char *volname, afs_uint32 * volid)
     dp = readdir(dirp);		/*read next entry in the directory */
     if (dp) {
 	if ((dp->d_name[0] == 'V') && !strcmp(&(dp->d_name[11]), VHDREXT)) {
-	    *volid = ExtractVolId(dp->d_name);
+	    *volid = VolumeNumber(dp->d_name);
 	    strcpy(volname, dp->d_name);
 	    return 0;		/*return the name of the file representing a volume */
 	} else {
